@@ -1,8 +1,5 @@
-/* $Id: cfile.h,v 1.14 2002/08/02 19:26:55 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
@@ -32,10 +29,10 @@ YAZ_BEGIN_CDECL
 #define HASH_BUCKET 15
 
 struct CFile_ph_bucket {     /* structure on disc */
-    int no[HASH_BUCKET];     /* block number in original file */
-    int vno[HASH_BUCKET];    /* block number in shadow file */
-    int this_bucket;         /* this bucket number */
-    int next_bucket;         /* next bucket number */
+    zint no[HASH_BUCKET];    /* block number in original file */
+    zint vno[HASH_BUCKET];   /* block number in shadow file */
+    zint this_bucket;        /* this bucket number */
+    zint next_bucket;        /* next bucket number */
 };
 
 struct CFile_hash_bucket {
@@ -53,21 +50,21 @@ typedef struct CFile_struct
 {
     struct CFile_head {
         int state;               /* 1 = hash, 2 = flat */
-        int next_block;          /* next free block / last block */
+        zint next_block;          /* next free block / last block */
         int block_size;          /* mfile/bfile block size */
         int hash_size;           /* no of chains in hash table */
-        int first_bucket;        /* first hash bucket */
-        int next_bucket;         /* last hash bucket + 1 = first flat bucket */
-        int flat_bucket;         /* last flat bucket + 1 */
+        zint first_bucket;       /* first hash bucket */
+        zint next_bucket;        /* last hash bucket + 1 = first flat bucket */
+        zint flat_bucket;        /* last flat bucket + 1 */
     } head;
     MFile block_mf;
     MFile hash_mf;
-    int *array;
+    zint *array;
     struct CFile_hash_bucket **parray;
     struct CFile_hash_bucket *bucket_lru_front, *bucket_lru_back;
     int dirty;
-    int bucket_in_memory;
-    int max_bucket_in_memory;
+    zint bucket_in_memory;
+    zint max_bucket_in_memory;
     char *iobuf;
     MFile rmf;
     int  no_hits;
@@ -78,11 +75,18 @@ typedef struct CFile_struct
 int cf_close (CFile cf);
 CFile cf_open (MFile mf, MFile_area area, const char *fname, int block_size,
                int wflag, int *firstp);
-int cf_read (CFile cf, int no, int offset, int nbytes, void *buf);
-int cf_write (CFile cf, int no, int offset, int nbytes, const void *buf);
-void cf_unlink (CFile cf);
-void cf_commit (CFile cf);
+int cf_read (CFile cf, zint no, int offset, int nbytes, void *buf);
+int cf_write (CFile cf, zint no, int offset, int nbytes, const void *buf);
+int cf_commit (CFile cf) ZEBRA_GCC_ATTR((warn_unused_result));
 
 YAZ_END_CDECL
 
 #endif
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
