@@ -1,8 +1,5 @@
-/* $Id: d1_absyn.c,v 1.9 2003/06/12 18:20:24 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 #include <stdio.h>
@@ -25,13 +22,105 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <stdlib.h>
 #include <string.h>
 
-#include <yaz/oid.h>
 #include <yaz/log.h>
-#include <data1.h>
+#include <yaz/oid_db.h>
+#include <idzebra/data1.h>
+#include <idzebra/recctrl.h>
 #include <zebra_xpath.h>
+#include <d1_absyn.h>
 
 #define D1_MAX_NESTING  128
 
+struct data1_hash_table {
+    NMEM nmem;
+    int size;
+    struct data1_hash_entry **ar;
+};
+
+struct data1_hash_entry {
+    void *clientData;
+    char *str;
+    struct data1_hash_entry *next;
+};
+
+unsigned data1_hash_calc(struct data1_hash_table *ht, const char *str)
+{
+    unsigned v = 0;
+    assert(str);
+    while (*str)
+    {
+	if (*str >= 'a' && *str <= 'z')
+	    v = v*65509 + *str -'a'+10;
+	else if (*str >= 'A' && *str <= 'Z')
+	    v = v*65509 + *str -'A'+10;
+	else if (*str >= '0' && *str <= '9')
+	    v = v*65509 + *str -'0';
+	str++;
+    }
+    return v % ht->size;
+}
+
+struct data1_hash_table *data1_hash_open(int size, NMEM nmem)
+{
+    int i;
+    struct data1_hash_table *ht = nmem_malloc(nmem, sizeof(*ht));
+    ht->nmem = nmem;
+    ht->size = size;
+    if (ht->size <= 0)
+	ht->size = 29;
+    ht->ar = nmem_malloc(nmem, sizeof(*ht->ar) * ht->size);
+    for (i = 0; i<ht->size; i++)
+	ht->ar[i] = 0;
+    return ht;
+}
+
+void data1_hash_insert(struct data1_hash_table *ht, const char *str,
+		       void *clientData, int copy)
+{
+    char *dstr = copy ? nmem_strdup(ht->nmem, str) : (char*) str;
+    if (strchr(str, '?') || strchr(str, '.'))
+    {
+	int i;
+	for (i = 0; i<ht->size; i++)
+	{
+	    struct data1_hash_entry **he = &ht->ar[i];
+	    for (; *he && strcmp(str, (*he)->str); he = &(*he)->next)
+		;
+	    if (!*he)
+	    {
+		*he = nmem_malloc(ht->nmem, sizeof(**he));
+		(*he)->str = dstr;
+		(*he)->next = 0;
+	    }
+	    (*he)->clientData = clientData;
+	}
+    }
+    else
+    {
+	struct data1_hash_entry **he = &ht->ar[data1_hash_calc(ht, str)];
+	for (; *he && strcmp(str, (*he)->str); he = &(*he)->next)
+	    ;
+	if (!*he)
+	{
+	    *he = nmem_malloc(ht->nmem, sizeof(**he));
+	    (*he)->str = dstr;
+	    (*he)->next = 0;
+	}
+	(*he)->clientData = clientData;
+    }
+}
+
+void *data1_hash_lookup(struct data1_hash_table *ht, const char *str)
+{
+    struct data1_hash_entry **he = &ht->ar[data1_hash_calc(ht, str)];
+    
+    for (; *he && yaz_matchstr(str, (*he)->str); he = &(*he)->next)
+	;
+    if (*he)
+	return (*he)->clientData;
+    return 0;
+}
+
 struct data1_systag {
     char *name;
     char *value;
@@ -52,17 +141,29 @@ struct data1_attset_cache_info
     data1_attset_cache next;
 };
 
+data1_element *data1_mk_element(data1_handle dh)
+{
+    data1_element *e = nmem_malloc(data1_nmem_get(dh), sizeof(*e));
+    e->name = 0;
+    e->tag = 0;
+    e->termlists = 0;
+    e->next = e->children = 0;
+    e->sub_name = 0;
+    e->hash = 0;
+    return e;
+}
+
 data1_absyn *data1_absyn_search (data1_handle dh, const char *name)
 {
     data1_absyn_cache p = *data1_absyn_cache_get (dh);
 
     while (p)
     {
-	if (!strcmp (name, p->name))
+	if (!yaz_matchstr (name, p->name))
 	    return p->absyn;
 	p = p->next;
     }
-    return NULL;
+    return 0;
 }
 /* *ostrich*
    We need to destroy DFAs, in xp_element (xelm) definitions 
@@ -80,10 +181,11 @@ void data1_absyn_destroy (data1_handle dh)
 	{
 	    data1_xpelement *xpe = abs->xp_elements;
 	    while (xpe) {
-		logf (LOG_DEBUG,"Destroy xp element %s",xpe->xpath_expr);
-		if (xpe->dfa) {  dfa_delete (&xpe->dfa); }
+		yaz_log (YLOG_DEBUG,"Destroy xp element %s",xpe->xpath_expr);
+		if (xpe->dfa) 
+                    dfa_delete (&xpe->dfa);
 		xpe = xpe->next;
-	    } 
+	    }
 	}
         p = p->next;
     }
@@ -102,7 +204,11 @@ void data1_absyn_trav (data1_handle dh, void *handle,
     }
 }
 
-data1_absyn *data1_absyn_add (data1_handle dh, const char *name)
+static data1_absyn *data1_read_absyn(data1_handle dh, const char *file,
+                                     enum DATA1_XPATH_INDEXING en);
+
+static data1_absyn *data1_absyn_add(data1_handle dh, const char *name,
+                                    enum DATA1_XPATH_INDEXING en)
 {
     char fname[512];
     NMEM mem = data1_nmem_get (dh);
@@ -110,20 +216,21 @@ data1_absyn *data1_absyn_add (data1_handle dh, const char *name)
     data1_absyn_cache p = (data1_absyn_cache)nmem_malloc (mem, sizeof(*p));
     data1_absyn_cache *pp = data1_absyn_cache_get (dh);
 
-    sprintf(fname, "%s.abs", name);
-    p->absyn = data1_read_absyn (dh, fname, 0);
-    p->name = nmem_strdup (mem, name);
+    sprintf(fname, "%.500s.abs", name);
+    p->absyn = data1_read_absyn(dh, fname, en);
+    p->name = nmem_strdup(mem, name);
     p->next = *pp;
     *pp = p;
     return p->absyn;
 }
 
-data1_absyn *data1_get_absyn (data1_handle dh, const char *name)
+data1_absyn *data1_get_absyn (data1_handle dh, const char *name,
+                              enum DATA1_XPATH_INDEXING en)
 {
     data1_absyn *absyn;
 
     if (!(absyn = data1_absyn_search (dh, name)))
-	absyn = data1_absyn_add (dh, name);
+	absyn = data1_absyn_add (dh, name, en);
     return absyn;
 }
 
@@ -133,51 +240,41 @@ data1_attset *data1_attset_search_name (data1_handle dh, const char *name)
 
     while (p)
     {
-	if (!strcmp (name, p->name))
+	if (!yaz_matchstr (name, p->name))
 	    return p->attset;
 	p = p->next;
     }
-    return NULL;
+    return 0;
 }
 
-data1_attset *data1_attset_search_id (data1_handle dh, int id)
+data1_attset *data1_attset_search_id(data1_handle dh, const Odr_oid *oid)
 {
     data1_attset_cache p = *data1_attset_cache_get (dh);
 
     while (p)
     {
-	if (id == p->attset->reference)
+	if (p->attset->oid && !oid_oidcmp(oid, p->attset->oid))
 	    return p->attset;
 	p = p->next;
     }
-    return NULL;
+    return 0;
 }
 
 data1_attset *data1_attset_add (data1_handle dh, const char *name)
 {
-    char fname[512], aname[512];
     NMEM mem = data1_nmem_get (dh);
     data1_attset *attset;
-
-    strcpy (aname, name);
-    sprintf(fname, "%s.att", name);
-    attset = data1_read_attset (dh, fname);
+    
+    attset = data1_read_attset (dh, name);
     if (!attset)
-    {
-	char *cp;
-	attset = data1_read_attset (dh, name);
-	if (attset && (cp = strrchr (aname, '.')))
-	    *cp = '\0';
-    }
-    if (!attset)
-	yaz_log (LOG_WARN|LOG_ERRNO, "Couldn't load attribute set %s", name);
+	yaz_log (YLOG_WARN|YLOG_ERRNO, "Couldn't load attribute set %s", name);
     else
     {
 	data1_attset_cache p = (data1_attset_cache)
 	    nmem_malloc (mem, sizeof(*p));
 	data1_attset_cache *pp = data1_attset_cache_get (dh);
 	
-	attset->name = p->name = nmem_strdup (mem, aname);
+	attset->name = p->name = nmem_strdup(mem, name);
 	p->attset = attset;
 	p->next = *pp;
 	*pp = p;
@@ -205,11 +302,15 @@ data1_esetname *data1_getesetbyname(data1_handle dh, data1_absyn *a,
     return 0;
 }
 
+/* we have multiple versions of data1_getelementbyname */
+#define DATA1_GETELEMENTBYTAGNAME_VERSION 1
+
 data1_element *data1_getelementbytagname (data1_handle dh, data1_absyn *abs,
 					  data1_element *parent,
 					  const char *tagname)
 {
     data1_element *r;
+    struct data1_hash_table *ht;
 
     /* It's now possible to have a data1 tree with no abstract syntax */
     if ( !abs )
@@ -220,6 +321,27 @@ data1_element *data1_getelementbytagname (data1_handle dh, data1_absyn *abs,
     else
 	r = parent->children;
 
+#if DATA1_GETELEMENTBYTAGNAME_VERSION==1
+    /* using hash search */
+    if (!r)
+	return 0;
+
+    ht = r->hash;
+    if (!ht)
+    {
+        /* build hash table (the first time) */
+	ht = r->hash = data1_hash_open(29, data1_nmem_get(dh));
+	for (; r; r = r->next)
+	{
+	    data1_name *n;
+	    
+	    for (n = r->tag->names; n; n = n->next)
+		data1_hash_insert(ht, n->name, r, 0);
+	}
+    }
+    return data1_hash_lookup(ht, tagname);
+#else
+    /* using linear search */
     for (; r; r = r->next)
     {
 	data1_name *n;
@@ -229,6 +351,7 @@ data1_element *data1_getelementbytagname (data1_handle dh, data1_absyn *abs,
 		return r;
     }
     return 0;
+#endif
 }
 
 data1_element *data1_getelementbyname (data1_handle dh, data1_absyn *absyn,
@@ -267,7 +390,7 @@ void fix_element_ref (data1_handle dh, data1_absyn *absyn, data1_element *e)
 	    if (sub_e)
 		e->children = sub_e->elements;
 	    else
-		yaz_log (LOG_WARN, "Unresolved reference to sub-elements %s",
+		yaz_log (YLOG_WARN, "Unresolved reference to sub-elements %s",
 		      e->sub_name);
 	}
     }
@@ -293,115 +416,149 @@ void fix_element_ref (data1_handle dh, data1_absyn *absyn, data1_element *e)
 
  */
 
-const char * mk_xpath_regexp (data1_handle dh, char *expr) 
+static const char * mk_xpath_regexp (data1_handle dh, const char *expr) 
 {
-    char *p = expr;
-    char *pp;
-    char *s;
+    const char *p = expr;
     int abs = 1;
-    int i;
-    int j;
-    int e=0;
-    int is_predicate = 0;
+    int e = 0;
+    char *stack[32];
+    char *res_p, *res = 0;
+    size_t res_size = 1;
     
-    static char *stack[32];
-    static char res[1024];
-    char *r = "";
-    
-    if (*p != '/') { return (""); }
+    if (*p != '/')
+	return ("");
     p++;
-    if (*p == '/') { abs=0; p++; }
-    
-    while (*p) {
-        i=0;
-        while (*p && !strchr("/",*p)) { 
-	  i++; p++; 
-	}
-        stack[e] = (char *) nmem_malloc (data1_nmem_get (dh), i+1);
+    if (*p == '/') 
+    { 
+	abs =0;
+	p++;
+    }
+    while (*p)
+    {
+	int is_predicate = 0;
+	char *s;
+	int i, j;
+        for (i = 0; *p && !strchr("/",*p); i++, p++)
+	    ;
+	res_size += (i+3); /* we'll add / between later .. */
+        stack[e] = (char *) nmem_malloc(data1_nmem_get(dh), i+1);
 	s = stack[e];
-	for (j=0; j< i; j++) {
-	  pp = p-i+j;
-	  if (*pp == '[') {
-	    is_predicate=1;
-	  }
-	  else if (*pp == ']') {
-	    is_predicate=0;
-	  }
-	  else {
-	    if (!is_predicate) {
-	      if (*pp == '*') 
-		 *s++ = '.';
-	      *s++ = *pp;
+	for (j = 0; j < i; j++)
+	{
+	    const char *pp = p-i+j;
+	    if (*pp == '[')
+		is_predicate=1;
+	    else if (*pp == ']')
+		is_predicate=0;
+	    else 
+	    {
+		if (!is_predicate) {
+		    if (*pp == '*') 
+			*s++ = '.';
+		    *s++ = *pp;
+		}
 	    }
-	  }
 	}
 	*s = 0;
         e++;
-        if (*p) {p++;}
+        if (*p)
+	    p++;
     }
-    e--;  p = &res[0]; i=0;
-    sprintf (p, "^"); p++;
-    while (e >= 0) {
-        /* !!! res size is not checked !!! */
-        sprintf (p, "%s/",stack[e]);
-        p += strlen(stack[e]) + 1;
-        e--;
+    res_p = res = nmem_malloc(data1_nmem_get(dh), res_size + 10);
+
+    if (stack[e-1][0] == '@')  /* path/@attr spec (leaf is attribute) */
+	strcpy(res_p, "/");
+    else
+	strcpy(res_p, "[^@]*/");  /* path .. (index all cdata below it) */
+    res_p = res_p + strlen(res_p);
+    while (--e >= 0) {
+	sprintf(res_p, "%s/", stack[e]);
+	res_p += strlen(stack[e]) + 1;
     }
-    if (!abs) { sprintf (p, ".*"); p+=2; }
-    sprintf (p, "$"); p++;
-    r = nmem_strdup (data1_nmem_get (dh), res);
-    yaz_log(LOG_DEBUG,"Got regexp: %s",r);
-    return (r);
+    if (!abs)
+    {
+	sprintf(res_p, ".*"); 
+	res_p += 2;
+    }
+    sprintf (res_p, "$");
+    res_p++;
+    yaz_log(YLOG_DEBUG, "Got regexp: %s", res);
+    return res;
 }
 
-/* *ostrich*
-
-   added arg xpelement... when called from xelm context, it's 1, saying
-   that ! means xpath, not element name as attribute name...
-
-   pop, 2002-12-13
- */
-static int parse_termlists (data1_handle dh, data1_termlist ***tpp,
-			    char *p, const char *file, int lineno,
-			    const char *element_name, data1_absyn *res,
-			    int xpelement)
+static int parse_termlists(data1_handle dh, data1_termlist ***tpp,
+                           char *cp, const char *file, int lineno,
+                           const char *element_name, data1_absyn *res,
+                           int xpelement,
+                           data1_attset *attset)
 {
     data1_termlist **tp = *tpp;
-    do
+    while(1)
     {
 	char attname[512], structure[512];
 	char *source;
-	int r;
-	
-	if (!(r = sscanf(p, "%511[^:,]:%511[^,]", attname,
-			 structure)))
+	int r, i;
+	int level = 0;
+	structure[0] = '\0';
+	for (i = 0; cp[i] && i<sizeof(attname)-1; i++)
+	    if (strchr(":,", cp[i]))
+		break;
+	    else
+		attname[i] = cp[i];
+	if (i == 0)
 	{
-	    yaz_log(LOG_WARN,
-		    "%s:%d: Syntax error in termlistspec '%s'",
-		    file, lineno, p);
-	    return -1;
+	    if (*cp)
+		yaz_log(YLOG_WARN,
+			"%s:%d: Syntax error in termlistspec '%s'",
+			file, lineno, cp);
+	    break;
 	}
+	attname[i] = '\0';
+	r = 1;
+	cp += i;
+	if (*cp == ':')
+	    cp++;
+
+	for (i = 0; cp[i] && i<sizeof(structure)-1; i++)
+	    if (level == 0 && strchr(",", cp[i]))
+		break;
+	    else
+	    {
+		structure[i] = cp[i];
+		if (cp[i] == '(')
+		    level++;
+		else if (cp[i] == ')')
+		    level--;
+	    }
+	structure[i] = '\0';
+	if (i)
+	    r = 2;
+	cp += i;
+	if (*cp)
+	    cp++;  /* skip , */
 
 	*tp = (data1_termlist *)
-	  nmem_malloc(data1_nmem_get(dh), sizeof(**tp));
+	    nmem_malloc(data1_nmem_get(dh), sizeof(**tp));
 	(*tp)->next = 0;
         
-	if (!xpelement) {
-            if (*attname == '!')
+        if (*attname == '!')
+        {
+            if (!xpelement && element_name)
                 strcpy(attname, element_name);
-	}
-	if (!((*tp)->att = data1_getattbyname(dh, res->attset,
-                                              attname))) {
-            if ((!xpelement) || (*attname != '!')) {
-                yaz_log(LOG_WARN,
-                        "%s:%d: Couldn't find att '%s' in attset",
-                        file, lineno, attname);
-                return -1;
-            } else {
-                (*tp)->att = 0;
+            else if (xpelement)
+                strcpy(attname, ZEBRA_XPATH_CDATA);
+        }
+        if (attset)
+        {
+            if (!data1_getattbyname(dh, attset, attname))
+            {
+                yaz_log(YLOG_WARN, "Index '%s' not found in attset(s)",
+                        attname);
             }
-	}
-        
+        }
+
+        (*tp)->index_name = nmem_strdup(data1_nmem_get(dh), attname);
+        assert (*(*tp)->index_name != '!');
 	if (r == 2 && (source = strchr(structure, ':')))
 	    *source++ = '\0';   /* cut off structure .. */
 	else
@@ -416,11 +573,38 @@ static int parse_termlists (data1_handle dh, data1_termlist ***tpp,
 		nmem_strdup (data1_nmem_get (dh), structure);
 	tp = &(*tp)->next;
     }
-    while ((p = strchr(p, ',')) && *(++p));
+
     *tpp = tp;
     return 0;
 }
 
+/* quinn
+ * Converts a 'melm' field[$subfield] pattern to a simple xpath
+ */
+static int melm2xpath(char *melm, char *buf)
+{
+    char *dollar;
+    char *field = melm;
+    char *subfield;
+    char *fieldtype;
+    if ((dollar = strchr(melm, '$'))) {
+	*dollar = '\0';
+	subfield = ++dollar;
+    } else
+	subfield = "";
+    if (field[0] == '0' && field[1] == '0')
+	fieldtype = "controlfield";
+    else
+	fieldtype = "datafield";
+    sprintf(buf, "/*/%s[@tag=\"%s\"]", fieldtype, field);
+    if (*subfield) 
+	sprintf(buf + strlen(buf), "/subfield[@code=\"%s\"]", subfield);
+    else if (field[0] != '0' || field[1] != '0')
+	strcat(buf, "/subfield");
+    yaz_log(YLOG_DEBUG, "Created xpath: '%s'", buf);
+    return 0;
+}
+
 const char *data1_systag_lookup(data1_absyn *absyn, const char *tag,
                                 const char *default_value)
 {
@@ -472,12 +656,28 @@ int read_absyn_line(FILE *f, int *lineno, char *line, int len,
     return argc;
 }
 
+data1_marctab *data1_absyn_getmarctab(data1_handle dh, data1_node *root)
+{
+    if (root->u.root.absyn)
+        return root->u.root.absyn->marc;
+    return 0;
+}
 
-data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
-                               int file_must_exist)
+data1_element *data1_absyn_getelements(data1_handle dh,
+                                       data1_node *root)
+{
+    if (root->u.root.absyn)
+        return root->u.root.absyn->main_elements;
+    return 0;
+}
+
+static data1_absyn *data1_read_absyn(data1_handle dh, const char *file,
+                                     enum DATA1_XPATH_INDEXING default_xpath)
 {
     data1_sub_elements *cur_elements = NULL;
-    data1_xpelement *cur_xpelement = NULL;
+    data1_xpelement **cur_xpelement = NULL;
+    data1_attset *attset_list = data1_empty_attset(dh);
+    data1_attset_child **attset_childp = &attset_list->children;
 
     data1_absyn *res = 0;
     FILE *f;
@@ -486,7 +686,6 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
     data1_maptab **maptabp;
     data1_marctab **marcp;
     data1_termlist *all = 0;
-    data1_attset_child **attset_childp;
     data1_tagset **tagset_childp;
     struct data1_systag **systagsp;
     int level = 0;
@@ -494,26 +693,19 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
     int argc;
     char *argv[50], line[512];
 
-    if (!(f = data1_path_fopen(dh, file, "r")))
-    {
-	yaz_log(LOG_WARN|LOG_ERRNO, "Couldn't open %s", file);
-        if (file_must_exist)
-            return 0;
-    }
+    f = data1_path_fopen(dh, file, "r");
     
     res = (data1_absyn *) nmem_malloc(data1_nmem_get(dh), sizeof(*res));
     res->name = 0;
-    res->reference = VAL_NONE;
+    res->oid = 0;
     res->tagset = 0;
     res->encoding = 0;
-    res->enable_xpath_indexing = (f ? 0 : 1);
+    res->xpath_indexing = 
+        (f ? DATA1_XPATH_INDEXING_DISABLE : default_xpath);
     res->systags = 0;
     systagsp = &res->systags;
     tagset_childp = &res->tagset;
 
-    res->attset = data1_empty_attset (dh);
-    attset_childp =  &res->attset->children;
-
     res->varset = 0;
     res->esetnames = 0;
     esetpp = &res->esetnames;
@@ -524,7 +716,8 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
     res->sub_elements = NULL;
     res->main_elements = NULL;
     res->xp_elements = NULL;
-    
+    cur_xpelement = &res->xp_elements;
+
     while (f && (argc = read_absyn_line(f, &lineno, line, 512, argv, 50)))
     {
 	char *cmd = *argv;
@@ -538,7 +731,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 
 	    if (argc < 4)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to elm", file, lineno);
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to elm", file, lineno);
 		continue;
 	    }
 	    path = argv[1];
@@ -569,17 +762,12 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    }
 	    if (i > level+1)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad level increase", file, lineno);
+		yaz_log(YLOG_WARN, "%s:%d: Bad level increase", file, lineno);
 		fclose(f);
 		return 0;
 	    }
 	    level = i;
-	    new_element = *ppl[level-1] = (data1_element *)
-		nmem_malloc(data1_nmem_get(dh), sizeof(*new_element));
-	    new_element->next = new_element->children = 0;
-	    new_element->tag = 0;
-	    new_element->termlists = 0;
-	    new_element->sub_name = 0;
+	    new_element = *ppl[level-1] = data1_mk_element(dh);
 	    
 	    tp = &new_element->termlists;
 	    ppl[level-1] = &new_element->next;
@@ -597,14 +785,14 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    {
 		if (!res->tagset)
 		{
-		    yaz_log(LOG_WARN, "%s:%d: No tagset loaded", file, lineno);
+		    yaz_log(YLOG_WARN, "%s:%d: No tagset loaded", file, lineno);
 		    fclose(f);
 		    return 0;
 		}
 		if (!(new_element->tag = data1_gettagbynum (dh, res->tagset,
 							    type, value)))
 		{
-		    yaz_log(LOG_WARN, "%s:%d: Couldn't find tag %s in tagset",
+		    yaz_log(YLOG_WARN, "%s:%d: Couldn't find tag %s in tagset",
 			 file, lineno, p);
 		    fclose(f);
 		    return 0;
@@ -630,7 +818,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    }
 	    else
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad element", file, lineno);
+		yaz_log(YLOG_WARN, "%s:%d: Bad element", file, lineno);
 		fclose(f);
 		return 0;
 	    }
@@ -638,9 +826,8 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    p = termlists;
 	    if (*p != '-')
 	    {
-		assert (res->attset);
-		
-		if (parse_termlists (dh, &tp, p, file, lineno, name, res, 0))
+		if (parse_termlists (dh, &tp, p, file, lineno, name, res, 0,
+                                     attset_list))
 		{
 		    fclose (f);
 		    return 0;
@@ -663,73 +850,92 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	   pop, 2003-01-17
 	*/
 
-	else if (!strcmp(cmd, "xelm")) {
+	else if (!strcmp(cmd, "xelm") || !strcmp(cmd, "melm")) {
 
 	    int i;
 	    char *p, *xpath_expr, *termlists;
 	    const char *regexp;
-	    struct DFA *dfa = dfa = dfa_init();
+	    struct DFA *dfa = 0;
 	    data1_termlist **tp;
+	    char melm_xpath[128];
+            data1_xpelement *xp_ele = 0;
+            data1_xpelement *last_match = 0;
             
-	    if (argc < 3)
+	    if (argc != 3)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to xelm", file, lineno);
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to %s",
+                        file, lineno, cmd);
 		continue;
 	    }
-	    xpath_expr = argv[1];
+
+	    if (!strcmp(cmd, "melm")) {
+		if (melm2xpath(argv[1], melm_xpath) < 0)
+		    continue;
+		xpath_expr = melm_xpath;
+	    } else {
+		xpath_expr = argv[1];
+	    }
 	    termlists = argv[2];
 	    regexp = mk_xpath_regexp(dh, xpath_expr);
-	    i = dfa_parse (dfa, &regexp);
-	    if (i || *regexp) {
-                yaz_log(LOG_WARN, "%s:%d: Bad xpath to xelm", file, lineno);
-                dfa_delete (&dfa);
-                continue;
-	    }
-            
-	    if (!cur_xpelement)
-	    {
-                cur_xpelement = (data1_xpelement *)
-		    nmem_malloc(data1_nmem_get(dh), sizeof(*cur_xpelement));
-		res->xp_elements = cur_xpelement;
-            } else {
-                cur_xpelement->next = (data1_xpelement *)
-                    nmem_malloc(data1_nmem_get(dh), sizeof(*cur_xpelement));
-                cur_xpelement = cur_xpelement->next;
-	    }
-	    cur_xpelement->next = NULL;
-	    cur_xpelement->xpath_expr = nmem_strdup(data1_nmem_get (dh), 
-						    xpath_expr); 
-	    
-	    dfa_mkstate (dfa);
-	    cur_xpelement->dfa = dfa;
 
-#ifdef ENHANCED_XELM 
-            cur_xpelement->xpath_len =
-                zebra_parse_xpath_str(xpath_expr, 
-                                      cur_xpelement->xpath, XPATH_STEP_COUNT,
-                                      data1_nmem_get(dh));
-            
-	    /*
-	    dump_xp_steps(cur_xpelement->xpath,cur_xpelement->xpath_len);
-	    */
+#if OPTIMIZE_MELM
+            /* get last of existing regulars with same regexp */
+            for (xp_ele = res->xp_elements; xp_ele; xp_ele = xp_ele->next)
+                if (!strcmp(xp_ele->regexp, regexp))
+                    last_match = xp_ele;
 #endif
-	    cur_xpelement->termlists = 0;
-	    tp = &cur_xpelement->termlists;
+            if (!last_match)
+            {
+                /* new regular expression . Parse + generate */
+                const char *regexp_ptr = regexp;
+
+                dfa = dfa_init();
+                i = dfa_parse (dfa, &regexp_ptr);
+                if (i || *regexp_ptr) {
+                    yaz_log(YLOG_WARN, "%s:%d: Bad xpath to xelm", file, lineno);
+                    dfa_delete (&dfa);
+                    continue;
+                }
+            }
+            *cur_xpelement = (data1_xpelement *)
+                nmem_malloc(data1_nmem_get(dh), sizeof(**cur_xpelement));
+            (*cur_xpelement)->next = 0;
+            (*cur_xpelement)->match_next = 0;
+            if (last_match)
+                last_match->match_next = *cur_xpelement;
+#if OPTIMIZE_MELM
+            (*cur_xpelement)->regexp = regexp;
+#endif
+	    (*cur_xpelement)->xpath_expr = nmem_strdup(data1_nmem_get (dh), 
+                                                       xpath_expr); 
+	    
+            if (dfa)
+                dfa_mkstate (dfa);
+            (*cur_xpelement)->dfa = dfa;
+            
+#ifdef ENHANCED_XELM 
+            (*cur_xpelement)->xpath_len =
+                zebra_parse_xpath_str(
+                    xpath_expr, 
+                    (*cur_xpelement)->xpath, XPATH_STEP_COUNT,
+                    data1_nmem_get(dh));
+#endif
+	    (*cur_xpelement)->termlists = 0;
+	    tp = &(*cur_xpelement)->termlists;
             
 	    /* parse termList definitions */
 	    p = termlists;
 	    if (*p != '-')
 	    {
-		assert (res->attset);
-		
 		if (parse_termlists (dh, &tp, p, file, lineno,
-                                     xpath_expr, res, 1))
+                                     xpath_expr, res, 1, attset_list))
 		{
 		    fclose (f);
 		    return 0;
 		}
 	        *tp = all; /* append any ALL entries to the list */
 	    }
+            cur_xpelement = &(*cur_xpelement)->next;
 	}
  	else if (!strcmp(cmd, "section"))
 	{
@@ -737,7 +943,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    
 	    if (argc < 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to section",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to section",
                         file, lineno);
 		continue;
 	    }
@@ -757,17 +963,17 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
         {
             if (argc != 2)
             {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to 'xpath' directive",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to 'xpath' directive",
 		     file, lineno);
 		continue;
             }
             if (!strcmp(argv[1], "enable"))
-                res->enable_xpath_indexing = 1;
+                res->xpath_indexing = DATA1_XPATH_INDEXING_ENABLE;
             else if (!strcmp (argv[1], "disable"))
-                res->enable_xpath_indexing = 0;
+                res->xpath_indexing = DATA1_XPATH_INDEXING_DISABLE;
             else
             {
-		yaz_log(LOG_WARN, "%s:%d: Expecting disable/enable "
+		yaz_log(YLOG_WARN, "%s:%d: Expecting disable/enable "
                         "after 'xpath' directive", file, lineno);
             }
         }
@@ -776,17 +982,18 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    data1_termlist **tp = &all;
 	    if (all)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Too many 'all' directives - ignored",
+		yaz_log(YLOG_WARN, "%s:%d: Too many 'all' directives - ignored",
 		     file, lineno);
 		continue;
 	    }
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to 'all' directive",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to 'all' directive",
 		     file, lineno);
 		continue;
 	    }
-	    if (parse_termlists (dh, &tp, argv[1], file, lineno, 0, res, 0))
+	    if (parse_termlists (dh, &tp, argv[1], file, lineno, 0, res, 0,
+                                 attset_list))
 	    {
 		fclose (f);
 		return 0;
@@ -796,7 +1003,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	{
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to name directive",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to name directive",
 		     file, lineno);
 		continue;
 	    }
@@ -808,41 +1015,44 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to reference",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to reference",
 		     file, lineno);
 		continue;
 	    }
 	    name = argv[1];
-	    if ((res->reference = oid_getvalbyname(name)) == VAL_NONE)
+            res->oid = yaz_string_to_oid_nmem(yaz_oid_std(),
+                                              CLASS_SCHEMA, name, 
+                                              data1_nmem_get(dh));
+            if (!res->oid)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Unknown tagset ref '%s'", 
+		yaz_log(YLOG_WARN, "%s:%d: Unknown tagset ref '%s'", 
 		     file, lineno, name);
 		continue;
 	    }
 	}
 	else if (!strcmp(cmd, "attset"))
 	{
-	    char *name;
-	    data1_attset *attset;
-	    
-	    if (argc != 2)
-	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to attset",
-		     file, lineno);
-		continue;
-	    }
-	    name = argv[1];
-	    if (!(attset = data1_get_attset (dh, name)))
-	    {
-		yaz_log(LOG_WARN, "%s:%d: Couldn't find attset  %s",
-		     file, lineno, name);
-		continue;
-	    }
-	    *attset_childp = (data1_attset_child *)
-		nmem_malloc (data1_nmem_get(dh), sizeof(**attset_childp));
-	    (*attset_childp)->child = attset;
-	    (*attset_childp)->next = 0;
-	    attset_childp = &(*attset_childp)->next;
+           char *name;
+           data1_attset *attset;
+           
+           if (argc != 2)
+           {
+               yaz_log(YLOG_WARN, "%s:%d: Bad # of args to attset",
+                    file, lineno);
+               continue;
+           }
+           name = argv[1];
+           if (!(attset = data1_get_attset (dh, name)))
+           {
+               yaz_log(YLOG_WARN, "%s:%d: Couldn't find attset  %s",
+                       file, lineno, name);
+               continue;
+           }
+           *attset_childp = (data1_attset_child *)
+               nmem_malloc (data1_nmem_get(dh), sizeof(**attset_childp));
+           (*attset_childp)->child = attset;
+           (*attset_childp)->next = 0;
+           attset_childp = &(*attset_childp)->next;
 	}
 	else if (!strcmp(cmd, "tagset"))
 	{
@@ -850,7 +1060,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    int type = 0;
 	    if (argc < 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args to tagset",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args to tagset",
 		     file, lineno);
 		continue;
 	    }
@@ -860,7 +1070,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    *tagset_childp = data1_read_tagset (dh, name, type);
 	    if (!(*tagset_childp))
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Couldn't load tagset %s",
+		yaz_log(YLOG_WARN, "%s:%d: Couldn't load tagset %s",
 		     file, lineno, name);
 		continue;
 	    }
@@ -872,14 +1082,14 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args in varset",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args in varset",
 		     file, lineno);
 		continue;
 	    }
 	    name = argv[1];
 	    if (!(res->varset = data1_read_varset (dh, name)))
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Couldn't load Varset %s",
+		yaz_log(YLOG_WARN, "%s:%d: Couldn't load Varset %s",
 		     file, lineno, name);
 		continue;
 	    }
@@ -890,7 +1100,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 
 	    if (argc != 3)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args in esetname",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args in esetname",
 		     file, lineno);
 		continue;
 	    }
@@ -905,7 +1115,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 		(*esetpp)->spec = 0;
 	    else if (!((*esetpp)->spec = data1_read_espec1 (dh, fname)))
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Espec-1 read failed for %s",
+		yaz_log(YLOG_WARN, "%s:%d: Espec-1 read failed for %s",
 		     file, lineno, fname);
 		continue;
 	    }
@@ -917,14 +1127,14 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # of args for maptab",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # of args for maptab",
                      file, lineno);
 		continue;
 	    }
 	    name = argv[1];
 	    if (!(*maptabp = data1_read_maptab (dh, name)))
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Couldn't load maptab %s",
+		yaz_log(YLOG_WARN, "%s:%d: Couldn't load maptab %s",
                      file, lineno, name);
 		continue;
 	    }
@@ -936,14 +1146,14 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	    
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # or args for marc",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # or args for marc",
 		     file, lineno);
 		continue;
 	    }
 	    name = argv[1];
 	    if (!(*marcp = data1_read_marctab (dh, name)))
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Couldn't read marctab %s",
+		yaz_log(YLOG_WARN, "%s:%d: Couldn't read marctab %s",
                      file, lineno, name);
 		continue;
 	    }
@@ -953,7 +1163,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	{
 	    if (argc != 2)
 	    {
-		yaz_log(LOG_WARN, "%s:%d: Bad # or args for encoding",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # or args for encoding",
 		     file, lineno);
 		continue;
 	    }
@@ -963,7 +1173,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
         {
             if (argc != 3)
             {
-		yaz_log(LOG_WARN, "%s:%d: Bad # or args for systag",
+		yaz_log(YLOG_WARN, "%s:%d: Bad # or args for systag",
 		     file, lineno);
 		continue;
             }
@@ -975,7 +1185,7 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
         }
 	else
 	{
-	    yaz_log(LOG_WARN, "%s:%d: Unknown directive '%s'", file, 
+	    yaz_log(YLOG_WARN, "%s:%d: Unknown directive '%s'", file, 
                     lineno, cmd);
 	    continue;
 	}
@@ -991,6 +1201,14 @@ data1_absyn *data1_read_absyn (data1_handle dh, const char *file,
 	fix_element_ref (dh, res, cur_elements->elements);
     }
     *systagsp = 0;
-    yaz_log (LOG_DEBUG, "%s: data1_read_absyn end", file);
     return res;
 }
+
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
