@@ -1,8 +1,5 @@
-/* $Id: d1_expout.c,v 1.2 2002/10/22 13:19:50 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 /*
@@ -30,7 +27,9 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 #include <yaz/log.h>
 #include <yaz/proto.h>
-#include <data1.h>
+#include <yaz/oid_db.h>
+#include <yaz/snprintf.h>
+#include <idzebra/data1.h>
 
 typedef struct {
     data1_handle dh;
@@ -47,12 +46,12 @@ static int is_numeric_tag (ExpHandle *eh, data1_node *c)
 	return 0;
     if (!c->u.tag.element)
     {
-	yaz_log(LOG_WARN, "Tag %s is local", c->u.tag.tag);
+	yaz_log(YLOG_WARN, "Tag %s is local", c->u.tag.tag);
 	return 0;
     }
     if (c->u.tag.element->tag->which != DATA1T_numeric)
     {
-	yaz_log(LOG_WARN, "Tag %s is not numeric", c->u.tag.tag);
+	yaz_log(YLOG_WARN, "Tag %s is not numeric", c->u.tag.tag);
 	return 0;
     }
     if (eh->select && !c->u.tag.node_selected)
@@ -113,31 +112,15 @@ static bool_t *f_bool(ExpHandle *eh, data1_node *c)
 static Odr_oid *f_oid(ExpHandle *eh, data1_node *c, oid_class oclass)
 {
     char oidstr[64];
-    int oid_this[20];
-    oid_value value_for_this;
 
     c = c->child;
     if (!is_data_tag (eh, c) || c->u.data.len > 63)
 	return 0;
-    sprintf(oidstr, "%.*s", c->u.data.len, c->u.data.data);
-    value_for_this = oid_getvalbyname(oidstr);
-    if (value_for_this == VAL_NONE)
-    {
-	Odr_oid *oid = odr_getoidbystr(eh->o, oidstr);
-	assert (oid);
-	return oid;
-    }
-    else
-    {
-	struct oident ident;
+    yaz_snprintf(oidstr, sizeof(oidstr)-1, 
+                 "%.*s", c->u.data.len, c->u.data.data);
 
-	ident.oclass = oclass;
-	ident.proto = PROTO_Z3950;
-	ident.value = value_for_this;
-	
-	oid_ent_to_oid (&ident, oid_this);
-    }
-    return odr_oiddup (eh->o, oid_this);
+    return yaz_string_to_oid_odr(yaz_oid_std(),
+                                 CLASS_GENERAL, oidstr, eh->o);
 }
 
 static Z_IntUnit *f_intunit(ExpHandle *eh, data1_node *c)
@@ -201,7 +184,7 @@ Odr_oid **f_oid_seq (ExpHandle *eh, data1_node *n, int *num, oid_class oclass)
 	    ++(*num);
     if (!*num)
 	return NULL;
-    res = (int **)odr_malloc (eh->o, sizeof(*res) * (*num));
+    res = (Odr_oid **)odr_malloc (eh->o, sizeof(*res) * (*num));
     for (c = n->child, i = 0 ; c; c = c->next)
 	if (is_numeric_tag (eh, c) == 1000)
 	    res[i++] = f_oid (eh, c, oclass);
@@ -1381,7 +1364,7 @@ Z_ExplainRecord *data1_nodetoexplain (data1_handle dh, data1_node *n,
     assert(n->which == DATA1N_root);
     if (strcmp(n->u.root.type, "explain"))
     {
-	yaz_log(LOG_WARN, "Attempt to convert a non-Explain record");
+	yaz_log(YLOG_WARN, "Attempt to convert a non-Explain record");
 	return 0;
     }
     for (n = n->child; n; n = n->next)
@@ -1415,6 +1398,14 @@ Z_ExplainRecord *data1_nodetoexplain (data1_handle dh, data1_node *n,
 	    return res;
 	}
     }
-    yaz_log(LOG_WARN, "No category in Explain record");
+    yaz_log(YLOG_WARN, "No category in Explain record");
     return 0;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
