@@ -1,8 +1,5 @@
-/* $Id: d1_grs.c,v 1.3 2003/03/27 21:57:01 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 /* converts data1 tree to GRS-1 record */
@@ -27,7 +24,7 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 
 #include <yaz/proto.h>
 #include <yaz/log.h>
-#include <data1.h>
+#include <d1_absyn.h>
 
 #define D1_VARIANTARRAY 20 /* fixed max length on sup'd variant-list. Lazy me */
 
@@ -94,7 +91,7 @@ static Z_Variant *make_variant(data1_node *n, int num, ODR o)
                     odr_strdup(o, p->u.variant.value);
 		break;
 	    default:
-		yaz_log(LOG_WARN, "Unable to handle value for variant %s",
+		yaz_log(YLOG_WARN, "Unable to handle value for variant %s",
 			p->u.variant.type->name);
 		return 0;
 	}
@@ -118,7 +115,7 @@ static int traverse_triples(data1_node *n, int level, Z_ElementMetaData *m,
 		    D1_VARIANTARRAY);
 	    else if (m->num_supportedVariants >= D1_VARIANTARRAY)
 	    {
-		yaz_log(LOG_WARN, "Too many variants (D1_VARIANTARRAY==%d)",
+		yaz_log(YLOG_WARN, "Too many variants (D1_VARIANTARRAY==%d)",
 			D1_VARIANTARRAY);
 		return -1;
 	    }
@@ -218,7 +215,7 @@ static Z_ElementData *nodetoelementdata(data1_handle dh, data1_node *n,
             *len += oid_oidlen(res->u.oid) * sizeof(int);
             break;
         default:
-            yaz_log(LOG_WARN, "Can't handle datatype.");
+            yaz_log(YLOG_WARN, "Can't handle datatype.");
             return 0;
 	}
     }
@@ -294,7 +291,7 @@ static Z_TaggedElement *nodetotaggedelement(data1_handle dh, data1_node *n,
     }
     else
     {
-	yaz_log(LOG_WARN, "Bad data.");
+	yaz_log(YLOG_WARN, "Bad data.");
 	return 0;
     }
 
@@ -392,7 +389,7 @@ Z_GenericRecord *data1_nodetogr(data1_handle dh, data1_node *n,
           data1_gettagbyname (dh, n->root->u.root.absyn->tagset,
                               "wellKnown")))
     {
-        yaz_log(LOG_WARN, "Unable to locate tag for 'wellKnown'");
+        yaz_log(YLOG_WARN, "Unable to locate tag for 'wellKnown'");
         wellknown_tag = odr_malloc(o, sizeof(*wellknown_tag));
         wellknown_tag->which = DATA1T_numeric;
         wellknown_tag->value.numeric = 19;
@@ -403,3 +400,11 @@ Z_GenericRecord *data1_nodetogr(data1_handle dh, data1_node *n,
     }
     return data1_nodetogr_r(dh, n, select, o, len, wellknown_tag);
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
