@@ -1,8 +1,5 @@
-/* $Id: d1_read.c,v 1.8 2004/07/26 13:51:42 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003,2004
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
@@ -29,10 +26,8 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <yaz/xmalloc.h>
 #include <yaz/yaz-util.h>
-#include <yaz/log.h>
-#include <data1.h>
+#include <d1_absyn.h>
 
 data1_node *data1_get_root_tag (data1_handle dh, data1_node *n)
 {
@@ -118,7 +113,7 @@ static void data1_init_node (data1_handle dh, data1_node *r, int type)
         r->u.preprocess.attributes = 0;
         break;
     default:
-	logf (LOG_WARN, "data_mk_node_type. bad type = %d\n", type);
+	yaz_log (YLOG_WARN, "data_mk_node_type. bad type = %d\n", type);
     }
 }
 
@@ -127,7 +122,6 @@ data1_node *data1_append_node (data1_handle dh, NMEM m, int type,
 {
     data1_node *r = (data1_node *)nmem_malloc(m, sizeof(*r));
     r->next = r->child = r->last_child = 0;
-    r->destroy = 0;
 
     r->parent = parent;
     if (!parent)
@@ -156,7 +150,6 @@ data1_node *data1_insert_node (data1_handle dh, NMEM m, int type,
 {
     data1_node *r = (data1_node *)nmem_malloc(m, sizeof(*r));
     r->next = r->child = r->last_child = 0;
-    r->destroy = 0;
     
     if (!parent)
         r->root = r;
@@ -174,27 +167,14 @@ data1_node *data1_insert_node (data1_handle dh, NMEM m, int type,
     return r;
 }
 
-void data1_free_tree (data1_handle dh, data1_node *t)
-{
-    data1_node *p = t->child, *pn;
-
-    while (p)
-    {
-	pn = p->next;
-	data1_free_tree (dh, p);
-	p = pn;
-    }
-    if (t->destroy)
-	(*t->destroy)(t);
-}
-
 data1_node *data1_mk_root (data1_handle dh, NMEM nmem, const char *name)
 {
-    data1_absyn *absyn = data1_get_absyn (dh, name);
+    data1_absyn *absyn = data1_get_absyn(dh, name, 1);
     data1_node *res;
+
     if (!absyn)
     {
-        yaz_log(LOG_WARN, "Unable to acquire abstract syntax " "for '%s'",
+        yaz_log(YLOG_WARN, "Unable to acquire abstract syntax " "for '%s'",
                 name); 
         /* It's now OK for a record not to have an absyn */
     }
@@ -207,7 +187,8 @@ data1_node *data1_mk_root (data1_handle dh, NMEM nmem, const char *name)
 void data1_set_root(data1_handle dh, data1_node *res,
                     NMEM nmem, const char *name)
 {
-    data1_absyn *absyn = data1_get_absyn (dh, name);
+    data1_absyn *absyn = data1_get_absyn(
+        dh, name, DATA1_XPATH_INDEXING_ENABLE);
 
     res->u.root.type = data1_insert_string (dh, res, nmem, name);
     res->u.root.absyn = absyn;
@@ -470,8 +451,8 @@ data1_node *data1_add_taggeddata (data1_handle dh, data1_node *root,
     return data1_add_insert_taggeddata (dh, at, tagname, m, 1, 0);
 }
 
-data1_node *data1_mk_tag_data_int (data1_handle dh, data1_node *at,
-                                   const char *tag, int num,
+data1_node *data1_mk_tag_data_zint (data1_handle dh, data1_node *at,
+                                   const char *tag, zint num,
                                    NMEM nmem)
 {
     data1_node *node_data;
@@ -481,11 +462,18 @@ data1_node *data1_mk_tag_data_int (data1_handle dh, data1_node *at,
 	return 0;
     node_data->u.data.what = DATA1I_num;
     node_data->u.data.data = node_data->lbuf;
-    sprintf (node_data->u.data.data, "%d", num);
+    sprintf (node_data->u.data.data, ZINT_FORMAT, num);
     node_data->u.data.len = strlen (node_data->u.data.data);
     return node_data;
 }
 
+data1_node *data1_mk_tag_data_int (data1_handle dh, data1_node *at,
+                                   const char *tag, int num,
+                                   NMEM nmem)
+{
+    return data1_mk_tag_data_zint(dh, at, tag, num, nmem);
+}
+
 data1_node *data1_mk_tag_data_oid (data1_handle dh, data1_node *at,
                                    const char *tag, Odr_oid *oid,
                                    NMEM nmem)
@@ -602,7 +590,6 @@ data1_xattr *data1_read_xattr (data1_handle dh, NMEM m,
     for (;;)
     {
 	data1_xattr *p;
-	int len;
 	while (*amp || (c && d1_isspace(c)))
 	    c = ampr (get_byte, fh, amp);
 	if (*amp == 0 && (c == 0 || c == '>' || c == '/'))
@@ -619,10 +606,7 @@ data1_xattr *data1_read_xattr (data1_handle dh, NMEM m,
 	    wrbuf_putc (wrbuf, c);
 	    c = ampr (get_byte, fh, amp);
 	}
-	wrbuf_putc (wrbuf, '\0');
-	len = wrbuf_len(wrbuf);
-	p->name = (char*) nmem_malloc (m, len);
-	strcpy (p->name, wrbuf_buf(wrbuf));
+	p->name = nmem_strdup (m, wrbuf_cstr(wrbuf));
 	if (c == '=')
 	{
 	    c = ampr (get_byte, fh, amp);
@@ -659,10 +643,7 @@ data1_xattr *data1_read_xattr (data1_handle dh, NMEM m,
 		    c = ampr (get_byte, fh, amp);
 	        }
             }
-	    wrbuf_putc (wrbuf, '\0');
-	    len = wrbuf_len(wrbuf);
-	    p->value = (char*) nmem_malloc (m, len);
-	    strcpy (p->value, wrbuf_buf(wrbuf));
+	    p->value = nmem_strdup(m, wrbuf_cstr(wrbuf));
 	}
     }
     *ch = c;
@@ -692,8 +673,7 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 	{
 	    data1_xattr *xattr;
 
-	    char tag[64];
-	    char args[256];
+	    char tag[256];
 	    int null_tag = 0;
 	    int end_tag = 0;
 	    size_t i = 0;
@@ -704,6 +684,33 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 		end_tag = 1;
 		c = ampr (get_byte, fh, &amp);
 	    }
+	    else if (amp == 0 && c == '?')
+	    {
+		int quote_mode = 0;
+		while ((c = ampr(get_byte, fh, &amp)))
+		{
+		    if (amp)
+			continue;
+		    if (quote_mode == 0)
+		    {
+			if (c == '"')
+			    quote_mode = c;
+			else if (c == '\'')
+			    quote_mode = c;
+			else if (c == '>')
+			{
+			    c = ampr(get_byte, fh, &amp);
+			    break;
+			}
+		    }
+		    else 
+		    {
+			if (amp == 0 && c == quote_mode)
+			    quote_mode = 0;
+		    }
+		}
+		continue;
+	    }
 	    else if (amp == 0 && c == '!')
 	    {
                 int c0, amp0;
@@ -771,7 +778,6 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 	    }
 	    tag[i] = '\0';
 	    xattr = data1_read_xattr (dh, m, get_byte, fh, wrbuf, &c, &amp);
-	    args[0] = '\0';
 	    if (amp == 0 && c == '/')
 	    {    /* <tag attrs/> or <tag/> */
 		null_tag = 1;
@@ -779,7 +785,7 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 	    }
 	    if (amp || c != '>')
 	    {
-		yaz_log(LOG_WARN, "d1: %d: Malformed tag", line);
+		yaz_log(YLOG_WARN, "d1: %d: Malformed tag", line);
 		return 0;
 	    }
 	    else
@@ -807,7 +813,7 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 		    }
 		    if (i != level)
 		    {
-			yaz_log (LOG_WARN, "%d: no begin tag for %s",
+			yaz_log (YLOG_WARN, "%d: no begin tag for %s",
 				 line, tag);
 			break;
 		    }
@@ -824,17 +830,20 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
                 }
 		continue;
 	    }	
-	    else if (!strcmp(tag, "var"))
+	    else if (!strcmp(tag, "var") 
+		     && xattr && xattr->next && xattr->next->next
+		     && xattr->value == 0 
+		     && xattr->next->value == 0
+		     && xattr->next->next->value == 0)
 	    {
-		char tclass[DATA1_MAX_SYMBOL], type[DATA1_MAX_SYMBOL];
+		/* <var class type value> */
+		const char *tclass = xattr->name;
+		const char *type = xattr->next->name;
+		const char *value = xattr->next->name;
 		data1_vartype *tp;
-		int val_offset;
 		
-		if (sscanf(args, "%s %s %n", tclass, type, &val_offset) != 2)
-		{
-		    yaz_log(LOG_WARN, "Malformed variant triple at '%s'", tag);
-		    continue;
-		}
+		yaz_log(YLOG_LOG, "Variant class=%s type=%s value=%s",
+			tclass, type, value);
 		if (!(tp =
 		      data1_getvartypebyct(dh,
 					   parent->root->u.root.absyn->varset,
@@ -864,7 +873,7 @@ data1_node *data1_read_nodex (data1_handle dh, NMEM m,
 		    res = data1_mk_node2 (dh, m, DATA1N_variant, parent);
 		    res->u.variant.type = tp;
 		    res->u.variant.value =
-			data1_insert_string (dh, res, m, args + val_offset);
+			data1_insert_string (dh, res, m, value);
 		}
 	    }
 	    else 
@@ -947,7 +956,7 @@ data1_node *data1_read_node (data1_handle dh, const char **buf, NMEM m)
     data1_node *node;
 
     node = data1_read_nodex(dh, m, getc_mem, (void *) (buf), wrbuf);
-    wrbuf_free (wrbuf, 1);
+    wrbuf_destroy(wrbuf);
     return node;
 }
 
@@ -992,34 +1001,12 @@ data1_node *data1_read_sgml (data1_handle dh, NMEM m, const char *buf)
 }
 
 
-static int conv_item (NMEM m, yaz_iconv_t t, 
-                      WRBUF wrbuf, char *inbuf, size_t inlen)
+static int conv_item(NMEM m, yaz_iconv_t t, 
+                     WRBUF wrbuf, char *inbuf, size_t inlen)
 {
-    wrbuf_rewind (wrbuf);
-    if (wrbuf->size < 10)
-        wrbuf_grow (wrbuf, 10);
-    for (;;)
-    {
-        char *outbuf = wrbuf->buf + wrbuf->pos;
-        size_t outlen = wrbuf->size - wrbuf->pos;
-        if (yaz_iconv (t, &inbuf, &inlen, &outbuf, &outlen) ==
-            (size_t)(-1) && yaz_iconv_error(t) != YAZ_ICONV_E2BIG)
-        {
-            /* bad data. stop and skip conversion entirely */
-            return -1;
-        }
-        else if (inlen == 0)
-        {   /* finished converting */
-            wrbuf->pos = wrbuf->size - outlen;
-            break;
-        }
-        else
-        {
-            /* buffer too small: make sure we expand buffer */
-            wrbuf->pos = wrbuf->size - outlen;
-            wrbuf_grow(wrbuf, 20);
-        }
-    }
+    wrbuf_rewind(wrbuf);
+    wrbuf_iconv_write(wrbuf, t, inbuf, inlen);
+    wrbuf_iconv_reset(wrbuf, t);
     return 0;
 }
 
@@ -1057,8 +1044,7 @@ static void data1_iconv_s (data1_handle dh, NMEM m, data1_node *n,
                         conv_item(m, t, wrbuf, p->value, strlen(p->value))
                         == 0)
                     {
-                        wrbuf_puts (wrbuf, "");
-                        p->value = nmem_strdup (m, wrbuf->buf);
+                        p->value = nmem_strdup(m, wrbuf_cstr(wrbuf));
                     }
                 }
             }
@@ -1097,22 +1083,54 @@ const char *data1_get_encoding (data1_handle dh, data1_node *n)
 }
 
 int data1_iconv (data1_handle dh, NMEM m, data1_node *n,
-                  const char *tocode, 
-                  const char *fromcode)
+                 const char *tocode, 
+                 const char *fromcode)
 {
     if (yaz_matchstr (tocode, fromcode))
     {
         WRBUF wrbuf = wrbuf_alloc();
-        yaz_iconv_t t = yaz_iconv_open (tocode, fromcode);
+        yaz_iconv_t t = yaz_iconv_open(tocode, fromcode);
         if (!t)
+	{
+            wrbuf_destroy(wrbuf);
             return -1;
-        data1_iconv_s (dh, m, n, t, wrbuf, tocode);
-        yaz_iconv_close (t);
-        wrbuf_free (wrbuf, 1);
+	}
+        data1_iconv_s(dh, m, n, t, wrbuf, tocode);
+        yaz_iconv_close(t);
+        wrbuf_destroy(wrbuf);
     }
     return 0;
 }
 
+void data1_chop_text(data1_handle dh, NMEM m, data1_node *n)
+{
+    for (; n; n = n->next)
+    {
+        if (n->which == DATA1N_data)
+        {
+            
+            int sz = n->u.data.len;
+            const char *ndata = n->u.data.data;
+            int off = 0;
+            
+            for (off = 0; off < sz; off++)
+                if (!d1_isspace(ndata[off]))
+                    break;
+            sz = sz - off;
+            ndata += off;
+            
+            while (sz && d1_isspace(ndata[sz - 1]))
+                sz--;
+
+            n->u.data.data = nmem_malloc(m, sz);
+            n->u.data.len = sz;
+            memcpy(n->u.data.data, ndata, sz);
+		
+        }
+        data1_chop_text(dh, m, n->child);
+    }
+}
+
 void data1_concat_text(data1_handle dh, NMEM m, data1_node *n)
 {
     for (; n; n = n->next)
@@ -1142,3 +1160,12 @@ void data1_concat_text(data1_handle dh, NMEM m, data1_node *n)
         data1_concat_text(dh, m, n->child);
     }
 }
+
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
