@@ -1,8 +1,5 @@
-/* $Id: d1_sutrs.c,v 1.2 2002/10/22 13:19:50 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,14 +12,14 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 /* converts data1 tree to SUTRS record */
 
-#include <data1.h>
+#include <idzebra/data1.h>
 
 #define NTOBUF_INDENT   2
 #define NTOBUF_MARGIN 75
@@ -149,3 +146,11 @@ char *data1_nodetobuf (data1_handle dh, data1_node *n, int select, int *len)
     *len = wrbuf_len(b);
     return wrbuf_buf(b);
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
