@@ -1,8 +1,5 @@
-/* $Id: dfa.c,v 1.30 2003/06/18 21:32:44 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
@@ -28,12 +25,10 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <string.h>
 #include <ctype.h>
 
-#include <zebrautl.h>
+#include <idzebra/util.h>
 #include "dfap.h"
 #include "imalloc.h"
 
-#define DFA_OPEN_RANGE 1
-
 #define CAT     16000
 #define OR      16001
 #define STAR    16002
@@ -50,8 +45,8 @@ struct Tnode {
     } u;
     unsigned pos : 15;        /* CAT/OR/STAR/EPSILON or non-neg. position */
     unsigned nullable : 1;
-    Set firstpos;             /* first positions */
-    Set lastpos;              /* last positions */
+    DFASet firstpos;             /* first positions */
+    DFASet lastpos;              /* last positions */
 };
 
 struct Tblock {               /* Tnode bucket (block) */
@@ -78,7 +73,7 @@ static void
     init_pos       (struct DFA_parse *parse_info), 
     del_pos        (struct DFA_parse *parse_info),
     mk_dfa_tran    (struct DFA_parse *parse_info, struct DFA_states *dfas),
-    add_follow     (struct DFA_parse *parse_info, Set lastpos, Set firstpos),
+    add_follow     (struct DFA_parse *parse_info, DFASet lastpos, DFASet firstpos),
     dfa_trav       (struct DFA_parse *parse_info, struct Tnode *n),
     init_followpos (struct DFA_parse *parse_info),
     pr_tran        (struct DFA_parse *parse_info, struct DFA_states *dfas),
@@ -357,7 +352,7 @@ static int nextchar_set (struct DFA_parse *parse_info, int *esc)
 
 static int read_charset (struct DFA_parse *parse_info)
 {
-    int i, ch0, ch1, esc0, esc1, cc = 0;
+    int i, ch0, esc0, cc = 0;
     parse_info->look_chars = mk_BSet (&parse_info->charset);
     res_BSet (parse_info->charset, parse_info->look_chars);
 
@@ -367,8 +362,13 @@ static int read_charset (struct DFA_parse *parse_info)
         cc = 1;
         ch0 = nextchar_set (parse_info, &esc0);
     }
+    /**
+       ch0 is last met character 
+       ch1 is "next" char 
+    */
     while (ch0 != 0)
     {
+        int ch1, esc1;
         if (!esc0 && ch0 == ']')
             break;
 	if (!esc0 && ch0 == '-')
@@ -380,16 +380,23 @@ static int read_charset (struct DFA_parse *parse_info)
 	}
 	else
 	{
-	    if (parse_info->cmap)
-	    {
-		const char **mapto;
-		char mapfrom[2];
-		const char *mcp = mapfrom;
-		mapfrom[0] = ch0;
-		mapto = (*parse_info->cmap)(parse_info->cmap_data, &mcp, 1);
-		assert (mapto);
-		ch0 = mapto[0][0];
-	    }
+            if (ch0 == 1)
+            {
+                ch0 = nextchar(parse_info, &esc0);
+            }
+            else
+            {
+                if (parse_info->cmap)
+                {
+                    const char **mapto;
+                    char mapfrom[2];
+                    const char *mcp = mapfrom;
+                    mapfrom[0] = ch0;
+                    mapto = parse_info->cmap(parse_info->cmap_data, &mcp, 1);
+                    assert (mapto);
+                    ch0 = mapto[0][0];
+                }
+            }
 	    add_BSet (parse_info->charset, parse_info->look_chars, ch0);
 	    ch1 = nextchar_set (parse_info, &esc1);
 	}
@@ -398,20 +405,16 @@ static int read_charset (struct DFA_parse *parse_info)
             int open_range = 0;
             if ((ch1 = nextchar_set (parse_info, &esc1)) == 0)
                 break;
-#if DFA_OPEN_RANGE
             if (!esc1 && ch1 == ']')
             {
                 ch1 = 255;
                 open_range = 1;
             }
-#else
-            if (!esc1 && ch1 == ']')
+            else if (ch1 == 1)
             {
-                add_BSet (parse_info->charset, parse_info->look_chars, '-');
-                break;
+                ch1 = nextchar(parse_info, &esc1);
             }
-#endif
-            if (!open_range && parse_info->cmap)
+            else if (parse_info->cmap)
             {
                 const char **mapto;
 		char mapfrom[2];
@@ -421,12 +424,12 @@ static int read_charset (struct DFA_parse *parse_info)
                 assert (mapto);
                 ch1 = mapto[0][0];
             }
-            for (i=ch0; ++i<=ch1;)
+            for (i = ch0; ++i <= ch1;)
                 add_BSet (parse_info->charset, parse_info->look_chars, i);
-            if (!open_range)
-                ch0 = nextchar_set (parse_info, &esc0);
-            else
+
+            if (open_range)
                 break;
+            ch0 = nextchar_set (parse_info, &esc0);
         }
         else
         {
@@ -459,7 +462,7 @@ static int map_l_char (struct DFA_parse *parse_info)
     
     parse_info->expr_ptr = (const unsigned char *) cp0;
     parse_info->look_ch = ((unsigned char **) mapto)[i][0];
-    logf (LOG_DEBUG, "map from %c to %d", parse_info->expr_ptr[-1], parse_info->look_ch);
+    yaz_log (YLOG_DEBUG, "map from %c to %d", parse_info->expr_ptr[-1], parse_info->look_ch);
     return L_CHAR;
 }
 
@@ -628,12 +631,12 @@ static void del_pos (struct DFA_parse *parse_info)
 }
 
 static void add_follow (struct DFA_parse *parse_info,
-			Set lastpos, Set firstpos)
+			DFASet lastpos, DFASet firstpos)
 {
     while (lastpos)
     {
         parse_info->followpos[lastpos->value] = 
-	    union_Set (parse_info->poset,
+	    union_DFASet (parse_info->poset,
 		       parse_info->followpos[lastpos->value], firstpos);
         lastpos = lastpos->next;
     }                                                            
@@ -642,7 +645,7 @@ static void add_follow (struct DFA_parse *parse_info,
 static void dfa_trav (struct DFA_parse *parse_info, struct Tnode *n)
 {
     struct Tnode **posar = parse_info->posar;
-    SetType poset = parse_info->poset;
+    DFASetType poset = parse_info->poset;
     
     switch (n->pos)
     {
@@ -650,21 +653,21 @@ static void dfa_trav (struct DFA_parse *parse_info, struct Tnode *n)
         dfa_trav (parse_info, n->u.p[0]);
         dfa_trav (parse_info, n->u.p[1]);
         n->nullable = n->u.p[0]->nullable & n->u.p[1]->nullable;
-        n->firstpos = mk_Set (poset);
-        n->firstpos = union_Set (poset, n->firstpos, n->u.p[0]->firstpos);
+        n->firstpos = mk_DFASet (poset);
+        n->firstpos = union_DFASet (poset, n->firstpos, n->u.p[0]->firstpos);
         if (n->u.p[0]->nullable)
-            n->firstpos = union_Set (poset, n->firstpos, n->u.p[1]->firstpos);
-        n->lastpos = mk_Set (poset);
-        n->lastpos = union_Set (poset, n->lastpos, n->u.p[1]->lastpos);
+            n->firstpos = union_DFASet (poset, n->firstpos, n->u.p[1]->firstpos);
+        n->lastpos = mk_DFASet (poset);
+        n->lastpos = union_DFASet (poset, n->lastpos, n->u.p[1]->lastpos);
         if (n->u.p[1]->nullable)
-            n->lastpos = union_Set (poset, n->lastpos, n->u.p[0]->lastpos);
+            n->lastpos = union_DFASet (poset, n->lastpos, n->u.p[0]->lastpos);
 
         add_follow (parse_info, n->u.p[0]->lastpos, n->u.p[1]->firstpos);
 
-        n->u.p[0]->firstpos = rm_Set (poset, n->u.p[0]->firstpos);
-        n->u.p[0]->lastpos = rm_Set (poset, n->u.p[0]->lastpos);
-        n->u.p[1]->firstpos = rm_Set (poset, n->u.p[1]->firstpos);
-        n->u.p[1]->lastpos = rm_Set (poset, n->u.p[1]->lastpos);
+        n->u.p[0]->firstpos = rm_DFASet (poset, n->u.p[0]->firstpos);
+        n->u.p[0]->lastpos = rm_DFASet (poset, n->u.p[0]->lastpos);
+        n->u.p[1]->firstpos = rm_DFASet (poset, n->u.p[1]->firstpos);
+        n->u.p[1]->lastpos = rm_DFASet (poset, n->u.p[1]->lastpos);
         if (debug_dfa_trav)
             printf ("CAT");
         break;
@@ -673,14 +676,14 @@ static void dfa_trav (struct DFA_parse *parse_info, struct Tnode *n)
         dfa_trav (parse_info, n->u.p[1]);
         n->nullable = n->u.p[0]->nullable | n->u.p[1]->nullable;
 
-        n->firstpos = merge_Set (poset, n->u.p[0]->firstpos,
+        n->firstpos = merge_DFASet (poset, n->u.p[0]->firstpos,
                                  n->u.p[1]->firstpos);
-        n->lastpos = merge_Set (poset, n->u.p[0]->lastpos,
+        n->lastpos = merge_DFASet (poset, n->u.p[0]->lastpos,
                                 n->u.p[1]->lastpos);
-        n->u.p[0]->firstpos = rm_Set (poset, n->u.p[0]->firstpos);
-        n->u.p[0]->lastpos = rm_Set (poset, n->u.p[0]->lastpos);
-        n->u.p[1]->firstpos = rm_Set (poset, n->u.p[1]->firstpos);
-        n->u.p[1]->lastpos = rm_Set (poset, n->u.p[1]->lastpos);
+        n->u.p[0]->firstpos = rm_DFASet (poset, n->u.p[0]->firstpos);
+        n->u.p[0]->lastpos = rm_DFASet (poset, n->u.p[0]->lastpos);
+        n->u.p[1]->firstpos = rm_DFASet (poset, n->u.p[1]->firstpos);
+        n->u.p[1]->lastpos = rm_DFASet (poset, n->u.p[1]->lastpos);
         if (debug_dfa_trav)
             printf ("OR");
         break;
@@ -704,18 +707,18 @@ static void dfa_trav (struct DFA_parse *parse_info, struct Tnode *n)
         break;
     case EPSILON:
         n->nullable = 1;
-        n->lastpos = mk_Set (poset);
-        n->firstpos = mk_Set (poset);
+        n->lastpos = mk_DFASet (poset);
+        n->firstpos = mk_DFASet (poset);
         if (debug_dfa_trav)
             printf ("EPSILON");
         break;
     default:
         posar[n->pos] = n;
         n->nullable = 0;
-        n->firstpos = mk_Set (poset);
-        n->firstpos = add_Set (poset, n->firstpos, n->pos);
-        n->lastpos = mk_Set (poset);
-        n->lastpos = add_Set (poset, n->lastpos, n->pos);
+        n->firstpos = mk_DFASet (poset);
+        n->firstpos = add_DFASet (poset, n->firstpos, n->pos);
+        n->lastpos = mk_DFASet (poset);
+        n->lastpos = add_DFASet (poset, n->lastpos, n->pos);
         if (debug_dfa_trav)
 	{
             if (n->u.ch[0] < 0)
@@ -737,20 +740,20 @@ static void dfa_trav (struct DFA_parse *parse_info, struct Tnode *n)
     {
         printf ("\n nullable : %c\n", n->nullable ? '1' : '0');
         printf (" firstpos :");
-        pr_Set (poset, n->firstpos);
+        pr_DFASet (poset, n->firstpos);
         printf (" lastpos  :");
-        pr_Set (poset, n->lastpos);
+        pr_DFASet (poset, n->lastpos);
     }
 }
 
 static void init_followpos (struct DFA_parse *parse_info)
 {
-    Set *fa;
+    DFASet *fa;
     int i;
     parse_info->followpos = fa =
-	(Set *) imalloc (sizeof(Set) * (1+parse_info->position));
+	(DFASet *) imalloc (sizeof(DFASet) * (1+parse_info->position));
     for (i = parse_info->position+1; --i >= 0; fa++)
-        *fa = mk_Set (parse_info->poset);
+        *fa = mk_DFASet (parse_info->poset);
 }
 
 static void mk_dfa_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
@@ -758,12 +761,12 @@ static void mk_dfa_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
     int i, j, c;
     int max_char;
     short *pos, *pos_i;
-    Set tran_set;
+    DFASet tran_set;
     int char_0, char_1;
     struct DFA_state *dfa_from, *dfa_to;
     struct Tnode **posar;
-    SetType poset;
-    Set *followpos;
+    DFASetType poset;
+    DFASet *followpos;
 
     assert (parse_info);
 
@@ -771,7 +774,7 @@ static void mk_dfa_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
     max_char = parse_info->charset->size;
     pos = (short *) imalloc (sizeof(*pos) * (parse_info->position+1));
 
-    tran_set = cp_Set (parse_info->poset, parse_info->root->firstpos);
+    tran_set = cp_DFASet (parse_info->poset, parse_info->root->firstpos);
     i = add_DFA_state (dfas, &tran_set, &dfa_from);
     assert (i);
     dfa_from->rule_no = 0;
@@ -814,7 +817,7 @@ static void mk_dfa_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
 
             char_1 = max_char;
                 
-            tran_set = mk_Set (poset);
+            tran_set = mk_DFASet (poset);
             for (pos_i = pos; (i = *pos_i) != -1; ++pos_i)
             {
                 if ((c=posar[i]->u.ch[0]) > char_0 && c <= char_1)
@@ -822,7 +825,7 @@ static void mk_dfa_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
                 else if ((c=posar[i]->u.ch[1]) >= char_0 && c < char_1)
                     char_1 = c;                    /* backward chunk */
                 if (posar[i]->u.ch[1] >= char_0 && posar[i]->u.ch[0] <= char_0)
-                    tran_set = union_Set (poset, tran_set, followpos[i]);
+                    tran_set = union_DFASet (poset, tran_set, followpos[i]);
             }
             if (tran_set)
             {
@@ -852,7 +855,7 @@ static void pr_tran (struct DFA_parse *parse_info, struct DFA_states *dfas)
             printf (" #%d", s->rule_nno);
 
         putchar (':');
-        pr_Set (parse_info->poset, s->set);
+        pr_DFASet (parse_info->poset, s->set);
         prev_no = -1;
         for (i=s->tran_no, tran=s->trans; --i >= 0; tran++)
         {
@@ -876,12 +879,12 @@ static void pr_verbose (struct DFA_parse *parse_info, struct DFA_states *dfas)
 {
     long i, j;
     int k;
-    printf ("%d/%d tree nodes used, %d bytes each\n",
-        parse_info->use_Tnode, parse_info->max_Tnode, sizeof (struct Tnode));
+    printf ("%d/%d tree nodes used, %ld bytes each\n",
+        parse_info->use_Tnode, parse_info->max_Tnode, (long) sizeof (struct Tnode));
     k = inf_BSetHandle (parse_info->charset, &i, &j);
-    printf ("%ld/%ld character sets, %d bytes each\n",
-            i/k, j/k, k*sizeof(BSetWord));
-    k = inf_SetType (parse_info->poset, &i, &j);
+    printf ("%ld/%ld character sets, %ld bytes each\n",
+            i/k, j/k, (long) k*sizeof(BSetWord));
+    k = inf_DFASetType (parse_info->poset, &i, &j);
     printf ("%ld/%ld poset items, %d bytes each\n", i, j, k);
     printf ("%d DFA states\n", dfas->no);
 }
@@ -894,7 +897,7 @@ static void pr_followpos (struct DFA_parse *parse_info)
     for (i=1; i <= parse_info->position; i++)
     {
         printf ("%3d:", i);
-        pr_Set (parse_info->poset, parse_info->followpos[i]);
+        pr_DFASet (parse_info->poset, parse_info->followpos[i]);
         putchar ('\t');
 
         if (posar[i]->u.ch[0] < 0)
@@ -1026,9 +1029,12 @@ static struct DFA_parse *dfa_parse_init (void)
     parse_info->rule = 0;
     parse_info->root = NULL;
 
+    /* initialize the anyset which by default does not include \n */
     parse_info->anyset = mk_BSet (&parse_info->charset);
     res_BSet (parse_info->charset, parse_info->anyset);
+    add_BSet (parse_info->charset, parse_info->anyset, '\n');
     com_BSet (parse_info->charset, parse_info->anyset);
+
     parse_info->use_Tnode = parse_info->max_Tnode = 0;
     parse_info->start = parse_info->end = NULL;
     parse_info->charMap = NULL;
@@ -1056,7 +1062,7 @@ static struct DFA_states *mk_dfas (struct DFA_parse *dfap, int poset_chunk)
     assert (poset_chunk > 10);
     assert (dfap);
 
-    parse_info->poset = mk_SetType (poset_chunk);
+    parse_info->poset = mk_DFASetType (poset_chunk);
     init_pos(parse_info);
     init_followpos(parse_info);
     assert (parse_info->root);
@@ -1074,7 +1080,7 @@ static struct DFA_states *mk_dfas (struct DFA_parse *dfap, int poset_chunk)
         pr_verbose (parse_info, dfas);
     del_pos(parse_info);
     del_followpos(parse_info);
-    rm_SetType (parse_info->poset);
+    rm_DFASetType (parse_info->poset);
     return dfas;
 }
 
@@ -1090,6 +1096,11 @@ struct DFA *dfa_init (void)
     return dfa;
 }
 
+void dfa_anyset_includes_nl(struct DFA *dfa)
+{
+    add_BSet (dfa->parse_info->charset, dfa->parse_info->anyset, '\n');
+}
+
 void dfa_set_cmap (struct DFA *dfa, void *vp,
 		   const char **(*cmap)(void *vp, const char **from, int len))
 {
@@ -1097,6 +1108,11 @@ void dfa_set_cmap (struct DFA *dfa, void *vp,
     dfa->parse_info->cmap_data = vp;
 }
 
+int dfa_get_last_rule (struct DFA *dfa)
+{
+    return dfa->parse_info->rule;
+}
+
 int dfa_parse (struct DFA *dfa, const char **pattern)
 {
     struct Tnode *top;
@@ -1106,12 +1122,6 @@ int dfa_parse (struct DFA *dfa, const char **pattern)
     assert (dfa->parse_info);
     parse_info = dfa->parse_info;
 
-    if (!parse_info->cmap)
-    {
-	res_BSet (parse_info->charset, parse_info->anyset);
-	add_BSet (parse_info->charset, parse_info->anyset, '\n');
-	com_BSet (parse_info->charset, parse_info->anyset);
-    }
     do_parse (parse_info, pattern, &top);
     if (parse_info->err_code)
         return parse_info->err_code;
@@ -1151,3 +1161,11 @@ void dfa_delete (struct DFA **dfap)
     ifree (*dfap);
     *dfap = NULL;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
