@@ -1,8 +1,5 @@
-/* $Id: dopen.c,v 1.9 2002/08/02 19:26:55 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,22 +12,22 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
 
 #include <sys/types.h>
 #include <fcntl.h>
-#ifndef WIN32
+#if HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
 
-#include <dict.h>
+#include "dict-p.h"
 
 static void common_init (Dict_BFile bf, int block_size, int cache)
 {
@@ -80,7 +77,10 @@ Dict_BFile dict_bf_open (BFiles bfs, const char *name, int block_size,
     dbf = (Dict_BFile) xmalloc (sizeof(*dbf));
     dbf->bf = bf_open (bfs, name, block_size, rw);
     if (!dbf->bf)
-        return NULL;
+    {
+        xfree(dbf);
+        return 0;
+    }
     common_init (dbf, block_size, cache);
     return dbf;
 }
@@ -89,3 +89,11 @@ void dict_bf_compact (Dict_BFile dbf)
 {
     dbf->compact_flag = 1;
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
