@@ -1,8 +1,5 @@
-/* $Id: open.c,v 1.19 2002/08/02 19:26:55 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 
@@ -26,66 +23,78 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <string.h>
 #include <stdio.h>
 
-#include <dict.h>
+#include "dict-p.h"
+
+void dict_clean(Dict dict)
+{
+    int page_size = dict->head.page_size;
+    void *head_buf;
+    int compact_flag = dict->head.compact_flag;
+
+    memset (dict->head.magic_str, 0, sizeof(dict->head.magic_str));
+    strcpy (dict->head.magic_str, DICT_MAGIC);
+    dict->head.last = 1;
+    dict->head.root = 0;
+    dict->head.freelist = 0;
+    dict->head.page_size = page_size;
+    dict->head.compact_flag = compact_flag;
+    
+    /* create header with information (page 0) */
+    if (dict->rw) 
+        dict_bf_newp (dict->dbf, 0, &head_buf, page_size);
+}
 
 Dict dict_open (BFiles bfs, const char *name, int cache, int rw,
-		int compact_flag)
+		int compact_flag, int page_size)
 {
     Dict dict;
     void *head_buf;
-    char resource_str[80];
-    int page_size;
 
     dict = (Dict) xmalloc (sizeof(*dict));
 
     if (cache < 5)
 	cache = 5;
-    sprintf (resource_str, "dict.%s.pagesize", name);
 
     dict->grep_cmap = NULL;
     page_size = DICT_DEFAULT_PAGESIZE;
     if (page_size < 2048)
     {
-        logf (LOG_WARN, "Resource %s was too small. Set to 2048",
-              resource_str);
+        yaz_log (YLOG_WARN, "Page size for dict %s %d<2048. Set to 2048",
+	      name, page_size);
         page_size = 2048;
     }
     dict->dbf = dict_bf_open (bfs, name, page_size, cache, rw);
     dict->rw = rw;
+    dict->no_split = 0;
+    dict->no_insert = 0;
+    dict->no_lookup = 0;
 
     if(!dict->dbf)
     {
-        logf (LOG_WARN, "Cannot open `%s'", name);
+        yaz_log (YLOG_WARN, "Cannot open `%s'", name);
         xfree (dict);
         return NULL;
     }
     if (dict_bf_readp (dict->dbf, 0, &head_buf) <= 0)
     {
-	memset (dict->head.magic_str, 0, sizeof(dict->head.magic_str));
-	strcpy (dict->head.magic_str, DICT_MAGIC);
-	dict->head.last = 1;
-	dict->head.root = 0;
-	dict->head.freelist = 0;
 	dict->head.page_size = page_size;
-	dict->head.compact_flag = compact_flag;
-	
-	/* create header with information (page 0) */
-        if (rw) 
-            dict_bf_newp (dict->dbf, 0, &head_buf, page_size);
+        dict->head.compact_flag = compact_flag;
+        dict_clean(dict);
     }
     else /* header was there, check magic and page size */
     {
 	memcpy (&dict->head, head_buf, sizeof(dict->head));
         if (strcmp (dict->head.magic_str, DICT_MAGIC))
         {
-            logf (LOG_WARN, "Bad magic of `%s'", name);
-            exit (1);
+            yaz_log (YLOG_WARN, "Bad magic of `%s'", name);
+	    dict_bf_close(dict->dbf);
+	    xfree(dict);
+	    return 0;
         }
         if (dict->head.page_size != page_size)
         {
-            logf (LOG_WARN, "Resource %s is %d and pagesize of `%s' is %d",
-                  resource_str, page_size, name, dict->head.page_size);
-	    return 0;
+            yaz_log (YLOG_WARN, "Page size for existing dict %s is %d. Current is %d",
+		  name, dict->head.page_size, page_size);
         }
     }
     if (dict->head.compact_flag)
@@ -107,3 +116,27 @@ int dict_strlen (const Dict_char *s)
 {
     return strlen((const char *) s);
 }
+
+zint dict_get_no_lookup(Dict dict)
+{
+    return dict->no_lookup;
+}
+
+zint dict_get_no_insert(Dict dict)
+{
+    return dict->no_insert;
+}
+
+zint dict_get_no_split(Dict dict)
+{
+    return dict->no_split;
+}
+
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
