@@ -1,8 +1,5 @@
-/* $Id: bset.h,v 1.4 2002/08/02 19:26:55 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,18 +12,17 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
-*/
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+*/
 
 #ifndef BSET_H
 #define BSET_H
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include <yaz/yconfig.h>
+
+YAZ_BEGIN_CDECL
 
 typedef unsigned short BSetWord;
 typedef BSetWord *BSet;
@@ -58,8 +54,14 @@ unsigned   hash_BSet      (BSetHandle *sh, BSet src);
 int        eq_BSet        (BSetHandle *sh, BSet dst, BSet src);
 void       pr_charBSet    (BSetHandle *sh, BSet src, void (*f)(int));
 
-#ifdef __cplusplus
-}
-#endif
+YAZ_END_CDECL
 
 #endif
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
