@@ -1,8 +1,5 @@
-/* $Id: direntz.h,v 1.8 2003/01/13 15:43:25 adam Exp $
-   Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,21 +12,18 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
-
-
-
 #ifdef WIN32
 /* make WIN32 version of dirent */
 #include <windows.h>
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include <yaz/yconfig.h>
+
+YAZ_BEGIN_CDECL
 
 struct dirent {
     char d_name[MAX_PATH];
@@ -41,9 +35,7 @@ DIR *opendir (const char *path);
 struct dirent *readdir (DIR *dd);
 void closedir (DIR *dd);
 
-#ifdef __cplusplus
-}
-#endif
+YAZ_END_CDECL
 
 #else
 /* include UNIX version */
@@ -51,3 +43,11 @@ void closedir (DIR *dd);
 #include <dirent.h>
 #endif
 
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
