@@ -1,8 +1,5 @@
-/* $Id: zebrash.c,v 1.27 2004/07/28 08:15:45 adam Exp $
-   Copyright (C) 2002,2003,2004
-   Index Data Aps
-
-This file is part of the Zebra server.
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
 
 Zebra is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free
@@ -15,9 +12,9 @@ FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 for more details.
 
 You should have received a copy of the GNU General Public License
-along with Zebra; see the file LICENSE.zebra.  If not, write to the
-Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
-02111-1307, USA.
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
 */
 
 /* 
@@ -28,7 +25,9 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <stdlib.h>
 #include <string.h> 
 #include <ctype.h>
-#include <unistd.h>  /* for isatty */
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 
 #if HAVE_READLINE_READLINE_H
 #include <readline/readline.h> 
@@ -37,12 +36,13 @@ Free Software Foundation, 59 Temple Place - Suite 330, Boston, MA
 #include <readline/history.h>
 #endif
 
-#include "zebraapi.h"
+#include <idzebra/api.h>
 #include <yaz/log.h>
 #include <yaz/proto.h>
 #include <yaz/sortspec.h>
 #include <yaz/options.h>
 #include <yaz/wrbuf.h>
+#include <yaz/oid_db.h>
 
 #define MAX_NO_ARGS 32
 #define MAX_OUT_BUFF 4096
@@ -62,6 +62,8 @@ ZebraHandle  zh=0;  /* the current session */
 /* time being, only one session works */
 int nextrecno=1;  /* record number to show next */
 static char *default_config = DEFAULTCONFIG;
+static int log_level=0;
+
 
 /**************************************
  * Help functions
@@ -189,7 +191,7 @@ static int cmd_zebra_open( char *args[], WRBUF outbuff)
     if (!zs)
 	wrbuf_puts(outbuff,"zebra seems not to have been started, "
 		   "trying anyway\n");
-    zh=zebra_open(zs);
+    zh = zebra_open(zs, 0);
     return 0; /* ok */
 }
 
@@ -210,10 +212,10 @@ static int cmd_quickstart( char *args[], WRBUF outbuff)
         rc=onecommand("yaz_log_file zebrash.log",outbuff,"");
     if (!rc)
         rc=onecommand("yaz_log_prefix ZebraSh", outbuff,"");
-    sprintf(tmp, "yaz_log_level 0x%x", LOG_DEFAULT_LEVEL | LOG_APP);
+    sprintf(tmp, "yaz_log_level 0x%x", YLOG_DEFAULT_LEVEL | log_level );
     if (!rc)
         rc=onecommand(tmp,outbuff,"");
-    logf(LOG_APP,"quickstart");
+    yaz_log(log_level,"quickstart");
     if (!zs)
         if (!rc)
             rc=onecommand("zebra_start",outbuff,"");
@@ -239,7 +241,7 @@ static int cmd_yaz_log_file( char *args[], WRBUF outbuff)
 
 static int cmd_yaz_log_level( char *args[], WRBUF outbuff)
 {
-    int  lev = defargint(args[1],LOG_DEFAULT_LEVEL);
+    int  lev = defargint(args[1],YLOG_DEFAULT_LEVEL);
     wrbuf_printf(outbuff, "setting yaz-log to level %d (ox%x)\n",lev,lev);
     yaz_log_init_level(lev);
     return 0; /* ok */
@@ -260,8 +262,8 @@ static int cmd_logf( char *args[], WRBUF outbuff)
     if (lev)
 	i=2;
     else
-	lev=LOG_LOG; /* this is in the default set!*/
-    logf( lev, restargs(args,i));
+	lev=YLOG_LOG; /* this is in the default set!*/
+    yaz_log( lev, restargs(args,i));
     return 0; /* ok */
 }
  
@@ -347,21 +349,21 @@ static int cmd_end_trans( char *args[], WRBUF outbuff)
 
 static int cmd_record_insert( char *args[], WRBUF outbuff)
 {
-    int sysno=0;
+    zint sysno = 0;
     int rc;
     char *rec=restargs(args,1);
     
-    rc = zebra_insert_record(zh,
+    rc = zebra_update_record(zh,
+                             action_insert,
 			     0,  /* record type */
 			     &sysno,
 			     0,  /* match */
 			     0,  /* fname */
 			     rec,
-			     strlen(rec),
-			     0);
+			     strlen(rec));
     if (0==rc)
     {
-        wrbuf_printf(outbuff,"ok sysno=%d\n",sysno);
+        wrbuf_printf(outbuff,"ok sysno=" ZINT_FORMAT "\n",sysno);
     }
     return rc;
 }
@@ -379,8 +381,12 @@ static int cmd_exchange_record( char *args[], WRBUF outbuff)
 	onecommand("help exchange_record", outbuff, "");
 	return -90;
     }
-    rc=zebra_admin_exchange_record(zh, rec, strlen(rec),
-        id, strlen(id), atoi(action));
+
+    rc = zebra_update_record(zh, action_update, 0 /* record_type */,
+                             0 /* sysno */,
+                             id /* match */,
+                             0 /* fname */,
+                             rec, strlen(rec));
     return rc;
 }
 
@@ -388,15 +394,15 @@ static int cmd_exchange_record( char *args[], WRBUF outbuff)
  * Searching and retrieving
  */
 
-static int cmd_search_pqf( char *args[], WRBUF outbuff)
+static int cmd_search_pqf(char *args[], WRBUF outbuff)
 {
-    int hits=0;
-    char *set=args[1];
-    char *qry=restargs(args,2);
+    zint hits = 0;
+    char *set = args[1];
+    char *qry = restargs(args,2);
     int rc;
-    rc=zebra_search_PQF(zh, qry, set, &hits);
+    rc = zebra_search_PQF(zh, qry, set, &hits);
     if (0==rc)
-        wrbuf_printf(outbuff,"%d hits found\n",hits);
+        wrbuf_printf(outbuff, ZINT_FORMAT " hits found\n", hits);
     return rc;
 }
 
@@ -404,21 +410,21 @@ static int cmd_find( char *args[], WRBUF outbuff)
 {
     char *setname=DEFAULTRESULTSET;
     int rc;
-    int hits=0;
-    WRBUF qry=wrbuf_alloc();
+    zint hits = 0;
+    WRBUF qry = wrbuf_alloc();
     if (0==strstr(args[0],"@attr"))
         wrbuf_puts(qry, "@attr 1=/ ");
     wrbuf_puts(qry,restargs(args,1));
     if (!zh)
 	onecommand("quickstart", outbuff, "");
-    wrbuf_printf(outbuff, "find %s\n",wrbuf_buf(qry));
-    rc=zebra_search_PQF(zh, wrbuf_buf(qry), setname, &hits);
+    wrbuf_printf(outbuff, "find %s\n",wrbuf_cstr(qry));
+    rc = zebra_search_PQF(zh, wrbuf_cstr(qry), setname, &hits);
     if (0==rc)
     {
-        wrbuf_printf(outbuff,"%d hits found\n",hits);
-        nextrecno=1;
+        wrbuf_printf(outbuff, ZINT_FORMAT " hits found\n", hits);
+        nextrecno = 1;
     }
-    wrbuf_free(qry,1);
+    wrbuf_destroy(qry);
     return rc;
 }
 
@@ -432,17 +438,16 @@ static int cmd_show( char *args[], WRBUF outbuff)
     ODR odr;
     Z_RecordComposition *pcomp=0;
     int i;
-    oid_value format;
 
-    odr=odr_createmem(ODR_ENCODE);
+    odr = odr_createmem(ODR_ENCODE);
     recs= odr_malloc(odr,sizeof(ZebraRetrievalRecord)*nrecs);
     rc =z_RecordComposition(odr, &pcomp, 0,"recordComposition");
-    format=oid_getvalbyname ("xml"); /*FIXME - let the user specify*/
+
     for (i=0;i<nrecs;i++)
         recs[i].position=start+i;
 
-    rc = zebra_records_retrieve (zh, odr, setname,
-				 pcomp, format, nrecs,recs);
+    rc = zebra_records_retrieve(zh, odr, setname,
+				pcomp, yaz_oid_recsyn_xml, nrecs,recs);
     if (0==rc)
     {
         for (i=0;i<nrecs;i++)
@@ -557,7 +562,7 @@ struct cmdstruct cmds[] = {
       "[prefix]",
       "Sets the log prefix",
       cmd_yaz_log_prefix},    
-    { "logf", 
+    { "yaz_log", 
       "[level] text...",
       "writes an entry in the log",
       cmd_logf},    
@@ -652,7 +657,7 @@ int onecommand(
     char *args[MAX_NO_ARGS];
     int nargs;
     char argbuf[MAX_ARG_LEN];
-    logf(LOG_APP,"%s",line);
+    yaz_log(log_level,"%s",line);
     strncpy(argbuf,line, MAX_ARG_LEN-1);
     argbuf[MAX_ARG_LEN-1]='\0'; /* just to be sure */
     /*memset(args,'\0',MAX_NO_ARGS*sizeof(char *));*/
@@ -692,7 +697,7 @@ int onecommand(
 	    return ((cmds[i].testfunc)(args,outbuff));
 	}
     wrbuf_printf(outbuff, "Unknown command '%s'. Try help\n",args[0]);
-    logf(LOG_APP,"Unknown command");
+    yaz_log(log_level,"Unknown command");
     return -90; 
 }
  
@@ -739,7 +744,7 @@ static int cmd_help( char *args[], WRBUF outbuff)
  
 /* If Zebra reports an error after an operation,
  * append it to the outbuff and log it */
-static void Zerrors ( WRBUF outbuff)
+static void Zerrors (WRBUF outbuff)
 {
     int ec;
     if (!zh)
@@ -747,13 +752,12 @@ static void Zerrors ( WRBUF outbuff)
     ec=zebra_errCode (zh);
     if (ec)
     {
-	logf(LOG_APP, "   Zebra error %d: %s, (%s)",
+	yaz_log(log_level, "   Zebra error %d: %s, (%s)",
 	     ec, zebra_errString (zh),
 	     zebra_errAdd (zh) );
 	wrbuf_printf(outbuff, "   Zebra error %d: %s, (%s)\n",
 		     ec, zebra_errString (zh),
 		     zebra_errAdd (zh) );
-	zebra_clearError(zh);
     }
 }
 
@@ -761,7 +765,7 @@ static void Zerrors ( WRBUF outbuff)
  * The shell
  */
  
-void shell()
+void shell(void)
 {
     int rc=0;
     WRBUF outbuff=wrbuf_alloc();
@@ -805,26 +809,26 @@ void shell()
 	/* get rid of \n in line */
 	if ((nl_cp = strchr(buf, '\n')))
 	    *nl_cp = '\0';
-	strncpy(prevout, wrbuf_buf(outbuff), MAX_OUT_BUFF);
+	strncpy(prevout, wrbuf_cstr(outbuff), MAX_OUT_BUFF);
         wrbuf_rewind(outbuff);
 	rc=onecommand(buf, outbuff, prevout);
 	if (rc==0)
 	{
 	    wrbuf_puts(outbuff, "   OK\n");
-	    logf(LOG_APP, "OK");
+	    yaz_log(log_level, "OK");
 	}
 	else if (rc>-90)
 	{
 	    wrbuf_printf(outbuff, "   command returned %d\n",rc);
 	} 
 	Zerrors(outbuff);
-	printf("%s\n", wrbuf_buf(outbuff));
+	printf("%s\n", wrbuf_cstr(outbuff));
     } /* while */
-    wrbuf_free(outbuff,1);
+    wrbuf_destroy(outbuff);
 } /* shell() */
 
 
-static void usage()
+static void usage(void)
 {
     printf ("usage:\n");
     printf ("zebrash [-c config]\n");
@@ -847,11 +851,22 @@ int main (int argc, char ** argv)
 	    break;
 	case 'h':
 	    usage();
+        /* FIXME - handle -v */
 	default:
 	    fprintf(stderr, "bad option %s\n", arg);
 	    usage();
 	}
     }
+    log_level=yaz_log_module_level("zebrash");
+
     shell();
     return 0;
 } /* main */
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
