@@ -1,8 +1,21 @@
-/*
- * $Id: testclient.c,v 1.1 2003/05/21 14:39:22 adam Exp $
- *
- * Z39.50 client specifically for Zebra testing.
- */
+/* This file is part of the Zebra server.
+   Copyright (C) 1995-2008 Index Data
+
+Zebra is free software; you can redistribute it and/or modify it under
+the terms of the GNU General Public License as published by the Free
+Software Foundation; either version 2, or (at your option) any later
+version.
+
+Zebra is distributed in the hope that it will be useful, but WITHOUT ANY
+WARRANTY; without even the implied warranty of MERCHANTABILITY or
+FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+for more details.
+
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+
+*/
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -27,7 +40,6 @@ int main(int argc, char **argv)
     char *target = 0;
     char *arg;
     int delay_sec = 0;
-    int expected_hits=-1;
     int ret;
     int retrieve_number = 0;
     int retrieve_offset = 0;
@@ -69,7 +81,7 @@ int main(int argc, char **argv)
         }
     }
 
-    if (!target || !target)
+    if (!target || !query)
     {
         printf ("%s: missing target/query\n", prog);
         printf ("usage:\n%s [options] target query \n", prog);
@@ -79,6 +91,7 @@ int main(int argc, char **argv)
         printf (" -o off       offset for records - counting from 0.\n");
         printf (" -f format    set record syntax. Default: none\n");
         printf (" -d sec       delay a number of seconds before exit.\n");
+        printf (" -c count     expect count hits, fail if not.\n");
         exit (3);
     }
     z = ZOOM_connection_new (target, 0);
@@ -98,11 +111,11 @@ int main(int argc, char **argv)
     }
     else
     {
-	printf ("Result count: %d\n", ZOOM_resultset_size(r));
+	printf ("Result count: %ld\n", (long) ZOOM_resultset_size(r));
 	if (check_count != -1 && check_count != ZOOM_resultset_size(r))
 	{
-	    printf("Wrong number of hits, expected %d, got %d\n",
-			    expected_hits, ZOOM_resultset_size(r) );
+	    printf("Wrong number of hits, expected %d, got %ld\n",
+			    check_count, (long) ZOOM_resultset_size(r) );
 	    exit(3);
 	}
     }
@@ -125,3 +138,11 @@ int main(int argc, char **argv)
     ZOOM_connection_destroy (z);
     exit (exit_code);
 }
+/*
+ * Local variables:
+ * c-basic-offset: 4
+ * indent-tabs-mode: nil
+ * End:
+ * vim: shiftwidth=4 tabstop=8 expandtab
+ */
+
